/**
 * <p>Copyright:Copyright(c) 2016</p>
 * <p>Company:上海中信信息发展股份有限公司</p>
 * <p>类更新历史信息</p>
 * @author 张二伟
 * @date 2017-03-19 14:58
 * @todo
 */

package com.cesgroup.cesmonitor.service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.ConcurrentHashMap;

import javax.mail.Message.RecipientType;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.cesgroup.cesmonitor.common.Util;
import com.cesgroup.cesmonitor.service.weixin.WxUser;
import com.cesgroup.cesmonitor.service.weixin.WxUtil;

/**
 * 媒体服务
 *
 * @author zhang.erwei
 * @date 2016-07-11
 *
 */
@Service("mediaService")
public class MediaService {

    @Value("${weixin.appId}")
    private String appId;

    @Value("${weixin.appsecret}")
    private String appsecret;

    //
    /**
     * 发件人的 邮箱
     */
    @Value("${mail.username}")
    public static String mailUsername = "zabbix_cesgroup@163.com";

    /**
     * 发件人的 密码,某些邮箱服务器为了增加邮箱本身密码的安全性， 给 SMTP
     * 客户端设置了独立密码（有的邮箱称为“授权码”）对于开启了独立密码的邮箱, 这里的邮箱密码必需使用这个独立密码（授权码）。
     */
    @Value("${mail.password}")
    public static String mailPassword = "zabbix123456";

    /**
     * 发件人邮箱的 SMTP 服务器地址, 必须准确, 不同邮件服务器地址不同, 一般(只是一般, 绝非绝对)格式为: smtp.xxx.com
     * 网易163邮箱的 SMTP 服务器地址为: smtp.163.com
     */
    @Value("${mail.smtp}")
    public static String mailSMTP = "smtp.163.com";

    /** 短信用户名 */
    @Value("${sm.userName}")
    private String userName = "";

    //密码
    @Value("${sm.password}")
    private String password = "";

    //玄武服务地址
    @Value("${sm.serverHost}")
    private String serverHost = "";

    //下行端口
    @Value("${sm.cmPort}")
    private int cmPort = 9080;

    //上行端口
    @Value("${sm.wsPort}")
    private int wsPort = 9070;

    protected Logger logger = LoggerFactory.getLogger(this.getClass());

    private Map<String, String> users = new ConcurrentHashMap<String, String>();

    /**
     * 微信，根据用户名称，获得用户openId
     *
     * @param sendTo
     *            用户名称
     * @return 用户openID
     */
    private synchronized String getOpenId(final String sendTo) {
        String openId = users.get(sendTo);
        if (openId == null) {
            for (String id : users.values()) {
                if (sendTo.equals(id)) {
                    openId = id;
                    break;
                }
            }
        }
        return openId;
    }

    /**
     * 发送微信消息
     *
     * @param sendTo
     *            用戶名称
     * @param subject
     *            主题
     * @param content
     *            内容
     * @return 用户openID
     */
    public synchronized String sendByWeixin(final String sendTo, final String subject, final String content) {
        String openId = this.getOpenId(sendTo);
        if (openId == null) {
            List<WxUser> list = WxUtil.getInstance(appId, appsecret).getUsers();
            for (WxUser user : list) {
                users.put(user.getNickname(), user.getOpenid());
            }
            openId = this.getOpenId(sendTo);
        }
        if (openId == null) {
            logger.warn("微信用戶不存在：" + sendTo);
            throw new RuntimeException("微信用戶不存在：" + sendTo);
        }
        return WxUtil.getInstance(appId, appsecret).sendAlertMessage(openId, subject, content,
            //Util.toStr(new Date(), Util.DF_TIME));
            Util.toStr(new Date(), new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")));

    }

    /**
     * 发送短信
     *
     * @param mobile
     *            获得接收者手机号码
     * @param content
     *            短信内容
     * @return
     *         短信
     */
    public synchronized String sendByShortMessage(final String mobile, final String content) {
        if (ShortMessageUtil.getInstance() == null) {
            ShortMessageUtil.init(userName, password, serverHost, cmPort, wsPort);
        }
        return ShortMessageUtil.getInstance().send(mobile, content);
    }

    /**
     * 发送邮件
     *
     * @param mailAddress
     *            收件人地址
     * @param subject
     *            主题
     * @param content
     *            邮件内容
     */
    public void sendByMail(final String mailAddress, final String subject, final String content) {
        try {
            // 1. 创建参数配置, 用于连接邮件服务器的参数配置
            Properties props = new Properties(); // 参数配置
            props.setProperty("mail.transport.protocol", "smtp"); // 使用的协议（JavaMail规范要求）
            props.setProperty("mail.smtp.host", mailSMTP); // 发件人的邮箱的 SMTP 服务器地址
            props.setProperty("mail.smtp.auth", "true"); // 需要请求认证
            // 2. 根据配置创建会话对象, 用于和邮件服务器交互
            Session session = Session.getDefaultInstance(props);
            session.setDebug(true); // 设置为debug模式, 可以查看详细的发送 log

            // 3. 创建一封邮件
            MimeMessage message = createMimeMessage(session, mailUsername, mailAddress, subject, content);

            // 4. 根据 Session 获取邮件传输对象
            Transport transport = session.getTransport();

            // 5. 使用 邮箱账号 和 密码 连接邮件服务器, 这里认证的邮箱必须与 message 中的发件人邮箱一致, 否则报错
            //
            //    PS_01: 成败的判断关键在此一句, 如果连接服务器失败, 都会在控制台输出相应失败原因的 log,
            //           仔细查看失败原因, 有些邮箱服务器会返回错误码或查看错误类型的链接, 根据给出的错误
            //           类型到对应邮件服务器的帮助网站上查看具体失败原因。
            //
            //    PS_02: 连接失败的原因通常为以下几点, 仔细检查代码:
            //           (1) 邮箱没有开启 SMTP 服务;
            //           (2) 邮箱密码错误, 例如某些邮箱开启了独立密码;
            //           (3) 邮箱服务器要求必须要使用 SSL 安全连接;
            //           (4) 请求过于频繁或其他原因, 被邮件服务器拒绝服务;
            //           (5) 如果以上几点都确定无误, 到邮件服务器网站查找帮助。
            //
            //    PS_03: 仔细看log, 认真看log, 看懂log, 错误原因都在log已说明。
            transport.connect(mailUsername, mailPassword);

            // 6. 发送邮件, 发到所有的收件地址, message.getAllRecipients() 获取到的是在创建邮件对象时添加的所有收件人, 抄送人, 密送人
            transport.sendMessage(message, message.getAllRecipients());

            // 7. 关闭连接
            transport.close();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    /**
     * 创建一封只包含文本的简单邮件
     *
     * @param session
     *            和服务器交互的会话
     * @param sendMail
     *            发件人邮箱
     * @param receiveMail
     *            收件人邮箱
     * @return 简单邮件
     * @throws Exception
     *             异常对象
     */
    public MimeMessage createMimeMessage(final Session session, final String sendMail, final String receiveMail,
        final String subject,
        final String content) {
        try {
            // 1. 创建一封邮件
            MimeMessage message = new MimeMessage(session);

            // 2. From: 发件人
            message.setFrom(new InternetAddress(sendMail, "智能监控平台", "UTF-8"));

            // 3. To: 收件人（可以增加多个收件人、抄送、密送）
            message.setRecipient(RecipientType.TO, new InternetAddress(receiveMail, "XX用户", "UTF-8"));

            // 4. Subject: 邮件主题
            message.setSubject(subject, "UTF-8");

            // 5. Content: 邮件正文（可以使用html标签）
            message.setContent(content, "text/html;charset=UTF-8");

            // 6. 设置发件时间
            message.setSentDate(new Date());

            // 7. 保存设置
            message.saveChanges();

            return message;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

}
